@@ -12,7 +12,6 @@ use syndicate::value::NestedValue;
 use tokio::net::TcpStream;
 
 use core::time::Duration;
-use tokio::time::interval;
 
 #[derive(Clone, Debug, StructOpt)]
 pub struct Config {
@@ -26,7 +25,7 @@ async fn main() -> Result<(), Box<dyn std::error::Error>> {
     let config = Config::from_args();
     let sturdyref = sturdy::SturdyRef::from_hex(&config.dataspace)?;
     let (i, o) = TcpStream::connect("127.0.0.1:8001").await?.into_split();
-    Actor::new(None).boot(syndicate::name!("consumer"), |t| {
+    Actor::top(None, |t| {
         relay::connect_stream(t, i, o, false, sturdyref, (), |_state, t, ds| {
             let consumer = syndicate::entity(0)
                 .on_message(|message_count, _t, m: AnyValue| {
@@ -44,18 +43,11 @@ async fn main() -> Result<(), Box<dyn std::error::Error>> {
                 observer: Arc::clone(&consumer),
             });
 
-            t.linked_task(syndicate::name!("tick"), async move {
-                let mut stats_timer = interval(Duration::from_secs(1));
-                loop {
-                    stats_timer.tick().await;
-                    let consumer = Arc::clone(&consumer);
-                    external_event(&Arc::clone(&consumer.underlying.mailbox),
-                                   &Account::new(syndicate::name!("account")),
-                                   Box::new(move |t| t.with_entity(
-                                       &consumer.underlying,
-                                       |t, e| e.message(t, AnyValue::new(true)))))?;
-                }
-            });
+            t.every(Duration::from_secs(1), move |t| {
+                consumer.message(t, &(), &AnyValue::new(true));
+                Ok(())
+            })?;
+
             Ok(None)
         });
         Ok(())
