@@ -4,8 +4,10 @@ use std::sync::Arc;
 
 use syndicate::actor::*;
 use syndicate::enclose;
+use syndicate::preserves::rec;
 use syndicate::schemas::service;
 use syndicate::supervise::{Supervisor, SupervisorConfiguration};
+use syndicate::trace;
 use syndicate::value::NestedValue;
 
 use tokio::io::AsyncRead;
@@ -21,7 +23,7 @@ use crate::schemas::external_services::*;
 use syndicate_macros::during;
 
 pub fn on_demand(t: &mut Activation, config_ds: Arc<Cap>, root_ds: Arc<Cap>) {
-    t.spawn(syndicate::name!("daemon"), move |t| {
+    t.spawn(Some(AnyValue::symbol("daemon_listener")), move |t| {
         Ok(during!(t, config_ds, language(), <run-service $spec: DaemonService>,
                    enclose!((config_ds, root_ds) move |t: &mut Activation| {
                        supervise_daemon(t, config_ds, root_ds, spec)
@@ -46,7 +48,7 @@ fn supervise_daemon(
             }));
         Supervisor::start(
             t,
-            syndicate::name!(parent: None, "daemon", id = ?spec.id),
+            Some(language().unparse(&spec)),
             SupervisorConfiguration::on_error_only(),
             enclose!((config_ds, spec) lifecycle::updater(config_ds, spec)),
             enclose!((config_ds, root_ds) move |t|
@@ -137,7 +139,7 @@ impl DaemonProcessSpec {
             },
             DaemonProcessSpec::OneShot { setup } => FullDaemonProcess {
                 process: Process::Simple(setup).elaborate(),
-                ready_on_start: ReadyOnStart::Absent,
+                ready_on_start: ReadyOnStart::Present { ready_on_start: false },
                 restart: RestartField::Present { restart: Box::new(RestartPolicy::OnError) },
                 protocol: ProtocolField::Absent,
             },
@@ -162,7 +164,6 @@ struct DaemonInstance {
     config_ds: Arc<Cap>,
     log_ds: Arc<Cap>,
     service: AnyValue,
-    name: tracing::Span,
     cmd: process::Command,
     announce_presumed_readiness: bool,
     unready_configs: Arc<Field<isize>>,
@@ -211,7 +212,6 @@ impl DaemonInstance {
                     t.facet(|t| {
                         let _ = t.prevent_inert_check();
                         counter::adjust(t, &self.completed_processes, 1);
-                        counter::adjust(t, &self.unready_configs, -1);
                         Ok(())
                     })?;
                     ()
@@ -224,25 +224,30 @@ impl DaemonInstance {
     fn log<R: 'static + Send + AsyncRead + Unpin>(
         &self,
         t: &mut Activation,
-        facet: FacetRef,
         pid: Option<u32>,
         r: R,
         kind: &str
-    ) {
-        let log_ds = self.log_ds.clone();
-        let service = self.service.clone();
-        let kind = AnyValue::symbol(kind);
-        let pid = match pid {
-            Some(n) => AnyValue::new(n),
-            None => AnyValue::symbol("unknown"),
-        };
-        t.spawn(syndicate::name!(parent: self.name.clone(), "log"), move |t| {
-            t.linked_task(tracing::Span::current(), async move {
+    ) -> ActorResult {
+        t.facet(|t| {
+            let facet = t.facet.clone();
+            let log_ds = self.log_ds.clone();
+            let service = self.service.clone();
+            let kind = AnyValue::symbol(kind);
+            let pid = match pid {
+                Some(n) => AnyValue::new(n),
+                None => AnyValue::symbol("unknown"),
+            };
+            let trace_collector = t.trace_collector();
+            t.linked_task(None, async move {
                 let mut r = BufReader::new(r);
+                let cause = trace_collector.as_ref().map(
+                    |_| trace::TurnCause::external(kind.value().as_symbol().unwrap()));
+                let account = Account::new(None, trace_collector);
                 loop {
                     let mut buf = Vec::new();
-                    if r.read_until(b'\n', &mut buf).await? == 0 {
-                        return Ok(LinkedTaskTermination::Normal);
+                    match r.read_until(b'\n', &mut buf).await {
+                        Ok(0) | Err(_) => break,
+                        Ok(_) => (),
                     }
                     let buf = match std::str::from_utf8(&buf) {
                         Ok(s) => AnyValue::new(s),
@@ -250,15 +255,14 @@ impl DaemonInstance {
                     };
                     let now = AnyValue::new(chrono::Utc::now().to_rfc3339());
                     if !facet.activate(
-                        Account::new(tracing::Span::current()),
-                        enclose!((pid, service, kind) |t| {
+                        &account, cause.clone(), enclose!((pid, service, kind) |t| {
                             log_ds.message(t, &(), &syndicate_macros::template!(
                                 "<log =now {
-                                             pid: =pid,
-                                             service: =service,
-                                             stream: =kind,
-                                             line: =buf,
-                                           }>"));
+                               pid: =pid,
+                               service: =service,
+                               stream: =kind,
+                               line: =buf,
+                             }>"));
                             Ok(())
                         }))
                     {
@@ -268,7 +272,8 @@ impl DaemonInstance {
                 Ok(LinkedTaskTermination::Normal)
             });
             Ok(())
-        });
+        })?;
+        Ok(())
     }
 
     fn start(mut self, t: &mut Activation) -> ActorResult {
@@ -287,7 +292,7 @@ impl DaemonInstance {
             let facet = t.facet.clone();
 
             if let Some(r) = child.stderr.take() {
-                self.log(t, facet.clone(), pid, r, "stderr");
+                self.log(t, pid, r, "stderr")?;
             }
 
             match self.protocol {
@@ -295,7 +300,7 @@ impl DaemonInstance {
                 Protocol::BinarySyndicate => self.relay_facet(t, &mut child, false)?,
                 Protocol::None => {
                     if let Some(r) = child.stdout.take() {
-                        self.log(t, facet.clone(), pid, r, "stdout");
+                        self.log(t, pid, r, "stdout")?;
                     }
                 }
             }
@@ -304,13 +309,17 @@ impl DaemonInstance {
                 counter::adjust(t, &self.unready_configs, -1);
             }
 
+            let trace_collector = t.trace_collector();
             t.linked_task(
-                syndicate::name!(parent: self.name.clone(), "wait"),
+                Some(rec![AnyValue::symbol("wait"), self.service.clone()]),
                 enclose!((facet) async move {
                     tracing::trace!("waiting for process exit");
                     let status = child.wait().await?;
                     tracing::debug!(?status);
-                    facet.activate(Account::new(syndicate::name!("instance-terminated")), |t| {
+                    let cause = trace_collector.as_ref().map(
+                        |_| trace::TurnCause::external("instance-terminated"));
+                    let account = Account::new(Some(AnyValue::symbol("instance-terminated")), trace_collector);
+                    facet.activate(&account, cause, |t| {
                         let m = if status.success() { None } else { Some(format!("{}", status)) };
                         self.handle_exit(t, m)
                     });
@@ -378,9 +387,10 @@ fn run(
         Ok(())
     }))?;
 
+    let trace_collector = t.trace_collector();
     enclose!((config_ds, unready_configs, completed_processes)
              during!(t, config_ds.clone(), language(), <daemon #(&service.id) $config>, {
-                 enclose!((spec, config_ds, root_ds, unready_configs, completed_processes)
+                 enclose!((spec, config_ds, root_ds, unready_configs, completed_processes, trace_collector)
                           |t: &mut Activation| {
                               tracing::debug!(?config, "new config");
                               counter::adjust(t, &unready_configs, 1);
@@ -391,7 +401,7 @@ fn run(
                                       tracing::info!(?config);
                                       let config = config.elaborate();
                                       let facet = t.facet.clone();
-                                      t.linked_task(syndicate::name!("subprocess"), async move {
+                                      t.linked_task(Some(AnyValue::symbol("subprocess")), async move {
                                           let mut cmd = config.process.build_command().ok_or("Cannot start daemon process")?;
 
                                           let announce_presumed_readiness = match config.ready_on_start {
@@ -432,7 +442,6 @@ fn run(
                                               config_ds,
                                               log_ds: root_ds,
                                               service: spec,
-                                              name: tracing::Span::current(),
                                               cmd,
                                               announce_presumed_readiness,
                                               unready_configs,
@@ -441,7 +450,10 @@ fn run(
                                               protocol,
                                           };
 
-                                          facet.activate(Account::new(syndicate::name!("instance-startup")), |t| {
+                                          let cause = trace_collector.as_ref().map(
+                                              |_| trace::TurnCause::external("instance-startup"));
+                                          let account = Account::new(Some(AnyValue::symbol("instance-startup")), trace_collector);
+                                          facet.activate(&account, cause, |t| {
                                               daemon_instance.start(t)
                                           });
                                           Ok(LinkedTaskTermination::KeepFacet)
